@@ -5,6 +5,8 @@
 // <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
 // option. This file may not be copied, modified, or distributed
 // except according to those terms.
+//
+// SPDX-License-Identifier: MIT/Apache-2.0
 
 use gst::glib;
 use gst::gst_debug;
@@ -130,7 +132,7 @@ impl Context {
                             } else {
                                 rav1e::prelude::FrameTypeOverride::No
                             },
-                            opaque: Some(Box::new(frame_number)),
+                            opaque: Some(rav1e::prelude::Opaque::new(frame_number)),
                         }),
                     ))
                 } else {
@@ -167,7 +169,7 @@ impl Context {
                             } else {
                                 rav1e::prelude::FrameTypeOverride::No
                             },
-                            opaque: Some(Box::new(frame_number)),
+                            opaque: Some(rav1e::prelude::Opaque::new(frame_number)),
                         }),
                     ))
                 } else {
@@ -215,7 +217,7 @@ impl ObjectImpl for Rav1Enc {
     fn properties() -> &'static [glib::ParamSpec] {
         static PROPERTIES: Lazy<Vec<glib::ParamSpec>> = Lazy::new(|| {
             vec![
-                glib::ParamSpec::new_uint(
+                glib::ParamSpecUInt::new(
                     "speed-preset",
                     "Speed Preset",
                     "Speed preset (10 fastest, 0 slowest)",
@@ -224,14 +226,14 @@ impl ObjectImpl for Rav1Enc {
                     DEFAULT_SPEED_PRESET,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_READY,
                 ),
-                glib::ParamSpec::new_boolean(
+                glib::ParamSpecBoolean::new(
                     "low-latency",
                     "Low Latency",
                     "Low Latency",
                     DEFAULT_LOW_LATENCY,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_READY,
                 ),
-                glib::ParamSpec::new_uint64(
+                glib::ParamSpecUInt64::new(
                     "min-key-frame-interval",
                     "Min Key Frame Interval",
                     "Min Key Frame Interval",
@@ -240,7 +242,7 @@ impl ObjectImpl for Rav1Enc {
                     DEFAULT_MIN_KEY_FRAME_INTERVAL,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_READY,
                 ),
-                glib::ParamSpec::new_uint64(
+                glib::ParamSpecUInt64::new(
                     "max-key-frame-interval",
                     "Max Key Frame Interval",
                     "Max Key Frame Interval",
@@ -249,7 +251,7 @@ impl ObjectImpl for Rav1Enc {
                     DEFAULT_MAX_KEY_FRAME_INTERVAL,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_READY,
                 ),
-                glib::ParamSpec::new_int(
+                glib::ParamSpecInt::new(
                     "bitrate",
                     "Bitrate",
                     "Bitrate",
@@ -258,7 +260,7 @@ impl ObjectImpl for Rav1Enc {
                     DEFAULT_BITRATE,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_READY,
                 ),
-                glib::ParamSpec::new_uint(
+                glib::ParamSpecUInt::new(
                     "quantizer",
                     "Quantizer",
                     "Quantizer",
@@ -267,7 +269,7 @@ impl ObjectImpl for Rav1Enc {
                     DEFAULT_QUANTIZER as u32,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_READY,
                 ),
-                glib::ParamSpec::new_uint(
+                glib::ParamSpecUInt::new(
                     "tile-cols",
                     "Tile Cols",
                     "Tile Cols",
@@ -276,7 +278,7 @@ impl ObjectImpl for Rav1Enc {
                     DEFAULT_TILE_COLS as u32,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_READY,
                 ),
-                glib::ParamSpec::new_uint(
+                glib::ParamSpecUInt::new(
                     "tile-rows",
                     "Tile Rows",
                     "Tile Rows",
@@ -285,7 +287,7 @@ impl ObjectImpl for Rav1Enc {
                     DEFAULT_TILE_ROWS as u32,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_READY,
                 ),
-                glib::ParamSpec::new_uint(
+                glib::ParamSpecUInt::new(
                     "tiles",
                     "Tiles",
                     "Tiles",
@@ -294,7 +296,7 @@ impl ObjectImpl for Rav1Enc {
                     DEFAULT_TILES as u32,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_READY,
                 ),
-                glib::ParamSpec::new_uint(
+                glib::ParamSpecUInt::new(
                     "threads",
                     "Threads",
                     "Threads",
@@ -408,6 +410,8 @@ impl ObjectImpl for Rav1Enc {
     }
 }
 
+impl GstObjectImpl for Rav1Enc {}
+
 impl ElementImpl for Rav1Enc {
     fn metadata() -> Option<&'static gst::subclass::ElementMetadata> {
         static ELEMENT_METADATA: Lazy<gst::subclass::ElementMetadata> = Lazy::new(|| {
@@ -424,35 +428,32 @@ impl ElementImpl for Rav1Enc {
 
     fn pad_templates() -> &'static [gst::PadTemplate] {
         static PAD_TEMPLATES: Lazy<Vec<gst::PadTemplate>> = Lazy::new(|| {
-            let sink_caps = gst::Caps::new_simple(
-                "video/x-raw",
-                &[
-                    (
-                        "format",
-                        &gst::List::new(&[
-                            &gst_video::VideoFormat::I420.to_str(),
-                            &gst_video::VideoFormat::Y42b.to_str(),
-                            &gst_video::VideoFormat::Y444.to_str(),
-                            &gst_video::VideoFormat::I42010le.to_str(),
-                            &gst_video::VideoFormat::I42210le.to_str(),
-                            &gst_video::VideoFormat::Y44410le.to_str(),
-                            &gst_video::VideoFormat::I42012le.to_str(),
-                            &gst_video::VideoFormat::I42212le.to_str(),
-                            &gst_video::VideoFormat::Y44412le.to_str(),
-                            // &gst_video::VideoFormat::Gray8.to_str(),
-                        ]),
+            let sink_caps = gst::Caps::builder("video/x-raw")
+                .field(
+                    "format",
+                    gst::List::new([
+                        gst_video::VideoFormat::I420.to_str(),
+                        gst_video::VideoFormat::Y42b.to_str(),
+                        gst_video::VideoFormat::Y444.to_str(),
+                        gst_video::VideoFormat::I42010le.to_str(),
+                        gst_video::VideoFormat::I42210le.to_str(),
+                        gst_video::VideoFormat::Y44410le.to_str(),
+                        gst_video::VideoFormat::I42012le.to_str(),
+                        gst_video::VideoFormat::I42212le.to_str(),
+                        gst_video::VideoFormat::Y44412le.to_str(),
+                        gst_video::VideoFormat::Gray8.to_str(),
+                    ]),
+                )
+                .field("width", gst::IntRange::new(1, std::i32::MAX))
+                .field("height", gst::IntRange::new(1, std::i32::MAX))
+                .field(
+                    "framerate",
+                    gst::FractionRange::new(
+                        gst::Fraction::new(0, 1),
+                        gst::Fraction::new(std::i32::MAX, 1),
                     ),
-                    ("width", &gst::IntRange::<i32>::new(1, std::i32::MAX)),
-                    ("height", &gst::IntRange::<i32>::new(1, std::i32::MAX)),
-                    (
-                        "framerate",
-                        &gst::FractionRange::new(
-                            gst::Fraction::new(0, 1),
-                            gst::Fraction::new(std::i32::MAX, 1),
-                        ),
-                    ),
-                ],
-            );
+                )
+                .build();
             let sink_pad_template = gst::PadTemplate::new(
                 "sink",
                 gst::PadDirection::Sink,
@@ -461,7 +462,10 @@ impl ElementImpl for Rav1Enc {
             )
             .unwrap();
 
-            let src_caps = gst::Caps::new_simple("video/x-av1", &[]);
+            let src_caps = gst::Caps::builder("video/x-av1")
+                .field("stream-format", "obu-stream")
+                .field("alignment", "tu")
+                .build();
             let src_pad_template = gst::PadTemplate::new(
                 "src",
                 gst::PadDirection::Src,
@@ -484,6 +488,15 @@ impl VideoEncoderImpl for Rav1Enc {
         Ok(())
     }
 
+    fn propose_allocation(
+        &self,
+        element: &Self::Type,
+        mut query: gst::query::Allocation<&mut gst::QueryRef>,
+    ) -> Result<(), gst::LoggableError> {
+        query.add_allocation_meta::<gst_video::VideoMeta>(None);
+        self.parent_propose_allocation(element, query)
+    }
+
     // For the colorimetry mapping below
     #[allow(clippy::wildcard_in_or_patterns)]
     fn set_format(
@@ -515,7 +528,7 @@ impl VideoEncoderImpl for Rav1Enc {
                     gst_video::VideoFormat::Y444
                     | gst_video::VideoFormat::Y44410le
                     | gst_video::VideoFormat::Y44412le => color::ChromaSampling::Cs444,
-                    // gst_video::VideoFormat::Gray8 => color::ChromaSampling::Cs400,
+                    gst_video::VideoFormat::Gray8 => color::ChromaSampling::Cs400,
                     _ => unreachable!(),
                 },
                 chroma_sample_position: match video_info.chroma_site() {
@@ -592,8 +605,8 @@ impl VideoEncoderImpl for Rav1Enc {
                 speed_settings: config::SpeedSettings::from_preset(settings.speed_preset as usize),
                 time_base: if video_info.fps() != gst::Fraction::new(0, 1) {
                     data::Rational {
-                        num: *video_info.fps().numer() as u64,
-                        den: *video_info.fps().denom() as u64,
+                        num: video_info.fps().numer() as u64,
+                        den: video_info.fps().denom() as u64,
                     }
                 } else {
                     data::Rational { num: 30, den: 1 }
@@ -625,7 +638,13 @@ impl VideoEncoderImpl for Rav1Enc {
         });
 
         let output_state = element
-            .set_output_state(gst::Caps::new_simple("video/x-av1", &[]), Some(state))
+            .set_output_state(
+                gst::Caps::builder("video/x-av1")
+                    .field("stream-format", "obu-stream")
+                    .field("alignment", "tu")
+                    .build(),
+                Some(state),
+            )
             .map_err(|_| gst::loggable_error!(CAT, "Failed to set output state"))?;
         element
             .negotiate(output_state)
